import * as i0 from '@angular/core';
import { Injectable, Directive, Optional, Input, InjectionToken, Inject, Injector, Component, EventEmitter, ViewContainerRef, ViewChild, Output, Renderer2, NgModule } from '@angular/core';
import { Subject, filter } from 'rxjs';
import * as i2 from '@angular/forms';
import { AbstractControl, FormGroup, FormControl, Validators, FormArray, ReactiveFormsModule } from '@angular/forms';
import { __decorate, __metadata } from 'tslib';
import * as i2$1 from '@angular/common';
import { CommonModule } from '@angular/common';
import * as i7 from '@angular/common/http';
import { HttpClient } from '@angular/common/http';

class ActionEventImpl {
    constructor(rawId, data) {
        this.rawId = rawId;
        this.matchId = rawId.replace(/\[\d+]/g, '');
        this.args = rawId.split('.')
            .map(idPart => {
            return idPart.split('[').map(arrayPart => {
                if (arrayPart.endsWith(']')) {
                    return Number(arrayPart.substring(0, arrayPart.length - 1));
                }
                else {
                    return arrayPart;
                }
            });
        })
            .reduce((last, current) => last.concat(current), []);
        this.data = data;
    }
}

class ActionService {
    constructor() {
        this._actions$ = new Subject();
    }
    get actions() {
        return this._actions$.asObservable();
    }
    actionEndWith(actionMatchId) {
        return this.actions.pipe(filter(event => event.matchId.endsWith(actionMatchId)));
    }
    emitAction(id, data) {
        this._actions$.next(new ActionEventImpl(id, data));
    }
}
ActionService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: ActionService, deps: [], target: i0.ɵɵFactoryTarget.Injectable });
ActionService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: ActionService });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: ActionService, decorators: [{
            type: Injectable
        }] });

const META_VALIDATOR_OPTIONS_KEY = 'AFF_VALIDATOR_OPTIONS_META';
const META_COMPONENT_OPTIONS_KEY = 'AFF_COMPONENT_OPTIONS_META';

function ActionControl(options) {
    return (target) => {
        const actionClass = target;
        actionClass[META_COMPONENT_OPTIONS_KEY] = Object.assign(Object.assign({}, actionClass[META_COMPONENT_OPTIONS_KEY]), { type: options.type, internalType: 'action' });
    };
}

class FromActionControlInternal extends AbstractControl {
    constructor() {
        super([], []);
        this._disabled = false;
    }
    get disabled() {
        return this._disabled;
    }
    disable() {
        this._disabled = true;
    }
    enable() {
        this._disabled = false;
    }
    setValue() {
        throw new Error('Method not supported on action.');
    }
    patchValue() {
        throw new Error('Method not supported on action.');
    }
    reset() {
        this.enable();
    }
}

class FormActionGroupInternal extends FormGroup {
    constructor(controls) {
        super(controls, [], []);
        this._disabled = false;
    }
    get disabled() {
        return this._disabled;
    }
    disable() {
        this._disabled = true;
    }
    enable() {
        this._disabled = false;
    }
    reset() {
        this._disabled = false;
    }
}

class FastFormGroup extends FormGroup {
    get questions() {
        return this._questions || [];
    }
    get mergedControls() {
        return Object.assign(Object.assign({}, this.actions), this.controls);
    }
    constructor(questions, controlFactory, options) {
        super({}, options);
        this.controlFactory = controlFactory;
        this.index = null;
        this.actions = {};
        this._questionChanges$ = new Subject();
        this._actions$ = new Subject();
        this.validateQuestions(questions);
        this.questionChanges = this._questionChanges$.asObservable();
        this._questions = questions;
        this.createChildControls();
        this.actionEvents = this._actions$.asObservable();
    }
    get(path) {
        var _a, _b, _c;
        const pathParts = path.split('.');
        const firstPath = pathParts.shift();
        if (firstPath === undefined) {
            return null;
        }
        if (pathParts.length === 0) {
            return (_a = this.mergedControls[firstPath]) !== null && _a !== void 0 ? _a : null;
        }
        else {
            return (_c = (_b = this.mergedControls[firstPath]) === null || _b === void 0 ? void 0 : _b.get(pathParts.join('.'))) !== null && _c !== void 0 ? _c : null;
        }
    }
    setQuestions(questions) {
        this._questions = questions;
        this.controls = {};
        this.createChildControls();
        this._questionChanges$.next(questions);
    }
    addControl(name, control, options) {
        if (control instanceof FromActionControlInternal || control instanceof FormActionGroupInternal) {
            this.actions[name] = control;
        }
        else {
            super.addControl(name, control, options);
        }
    }
    createChildControls() {
        this.controlFactory.createFromQuestions(this, this._questions);
    }
    validateQuestions(questions) {
        const ids = new Set();
        questions.forEach(q => {
            if (ids.has(q.id)) {
                throw new Error(`Duplicated form control id ${q.id} found.`);
            }
            ids.add(q.id);
        });
    }
}

class FastFormControl extends FormControl {
    constructor(question, value) {
        super(value);
        this.question = question;
        this.index = null;
    }
}

class ArrayIndexDirective {
    set affArrayIndex(index) {
        this.index = index;
    }
    constructor(group, control) {
        this.group = group;
        this.control = control;
    }
    ngOnChanges(changes) {
        var _a, _b;
        if (changes['affArrayIndex']) {
            if (((_a = this.group) === null || _a === void 0 ? void 0 : _a.form) instanceof FastFormGroup) {
                this.group.form.index = this.index;
            }
            if (((_b = this.control) === null || _b === void 0 ? void 0 : _b.form) instanceof FastFormControl) {
                this.control.form.index = this.index;
            }
        }
    }
}
ArrayIndexDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: ArrayIndexDirective, deps: [{ token: i2.FormGroupDirective, optional: true }, { token: i2.FormControlDirective, optional: true }], target: i0.ɵɵFactoryTarget.Directive });
ArrayIndexDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "15.2.10", type: ArrayIndexDirective, selector: "[affArrayIndex]", inputs: { affArrayIndex: "affArrayIndex" }, usesOnChanges: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: ArrayIndexDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[affArrayIndex]'
                }]
        }], ctorParameters: function () {
        return [{ type: i2.FormGroupDirective, decorators: [{
                        type: Optional
                    }] }, { type: i2.FormControlDirective, decorators: [{
                        type: Optional
                    }] }];
    }, propDecorators: { affArrayIndex: [{
                type: Input
            }] } });

class ActionGroupFactory {
    static create(children) {
        return new FormActionGroupInternal(children);
    }
}

class ActionControlFactory {
    static create() {
        return new FromActionControlInternal();
    }
}

/**
 * @deprecated Will be removed in version [3.0.0]. Use dependency
 * injection pattern instead.
 */
class BaseFormControlComponent {
}

/**
 * @deprecated Will be removed in version [3.0.0]. Use dependency
 * injection pattern instead.
 */
class BaseFormArrayComponent {
}

/**
 * @deprecated Will be removed in version [3.0.0]. Use dependency
 * injection pattern instead.
 */
class BaseFormGroupComponent {
}

function Control(options) {
    return (target) => {
        const controlComponent = target;
        controlComponent[META_COMPONENT_OPTIONS_KEY] = Object.assign(Object.assign({}, controlComponent[META_COMPONENT_OPTIONS_KEY]), { internalType: options.controlType === undefined ? 'control' : options.controlType, type: options.type, inline: options.inline });
    };
}

//TODO: Remove this and render with old render service and old control wrapper
class ControlWrapperV2 {
    constructor(_controlType, _initialState, _question, _arrayQuestion, _groupQuestions) {
        this._controlType = _controlType;
        this._initialState = _initialState;
        this._question = _question;
        this._arrayQuestion = _arrayQuestion;
        this._groupQuestions = _groupQuestions;
    }
    get controlType() {
        return this._controlType;
    }
    get initialState() {
        return this._initialState;
    }
    get question() {
        return this._question;
    }
    get arrayQuestion() {
        if (this._arrayQuestion) {
            return this._arrayQuestion;
        }
        else {
            throw new Error('No array question defined.');
        }
    }
    get groupQuestion() {
        if (this._groupQuestions) {
            return this._groupQuestions;
        }
        else {
            throw new Error('No group question defined.');
        }
    }
    static fromControl(initialState, opts) {
        return new ControlWrapperV2('control', initialState, opts, null, null);
    }
    static fromGroup(question, groupQuestions) {
        return new ControlWrapperV2('group', null, question, null, groupQuestions);
    }
    static fromArray(question, arrayQuestion) {
        return new ControlWrapperV2('array', null, question, arrayQuestion !== null && arrayQuestion !== void 0 ? arrayQuestion : null, null);
    }
}

class ValidatorDefinition {
    constructor(definition) {
        this.definition = definition;
        if (this.definition.includes(':')) {
            this.id = this.definition.split(':')[0];
            this.args = this.definition.split(':')[1].split(',');
        }
        else {
            this.id = this.definition;
            this.args = [];
        }
    }
}

const VALIDATORS = new InjectionToken('AFF_VALIDATORS');

class ValidatorRegistry {
    constructor(injector, validatorsPerModule) {
        this.injector = injector;
        this.validators = [];
        for (const vList of validatorsPerModule) {
            for (const v of (vList !== null && vList !== void 0 ? vList : [])) {
                this.validate(v);
                this.validators.push(v);
            }
        }
    }
    hasSyncValidator(id) {
        return !!this.getValidatorType(id, 'sync');
    }
    hasAsyncValidator(id) {
        return !!this.getValidatorType(id, 'async');
    }
    hasValidator(id, type) {
        return !!this.getValidatorType(id, type);
    }
    getSyncValidator(def) {
        return this.createValidator(def, 'sync');
    }
    getAsyncValidator(def) {
        return this.createValidator(def, 'async');
    }
    getValidator(def, type) {
        return this.createValidator(def, type);
    }
    createValidator(def, type) {
        const validatorType = this.getValidatorType(def.id, type);
        if (!validatorType) {
            throw new Error(`No ${type} validator registered with type [${def.id}]`);
        }
        const validatorFactory = this.injector.get(validatorType);
        return validatorFactory.createValidator(def.args);
    }
    getValidatorType(id, type) {
        return this.validators.find(validator => {
            const options = validator[META_VALIDATOR_OPTIONS_KEY];
            return options.type === type && options.id === id;
        });
    }
    validate(v) {
        if (!v[META_VALIDATOR_OPTIONS_KEY]) {
            throw new Error(`Validator must be decorated with [@Validator] decorator.`);
        }
    }
}

function toArray(option) {
    if (option) {
        if (option instanceof Array) {
            return option;
        }
        else {
            return [option];
        }
    }
    else {
        return [];
    }
}

class ValidatorFactoryService {
    constructor(injector, registeredValidators) {
        this.injector = injector;
        this.registry = new ValidatorRegistry(injector, registeredValidators !== null && registeredValidators !== void 0 ? registeredValidators : []);
    }
    createValidators(options) {
        const validators = [];
        if (!options) {
            return validators;
        }
        if (options.required) {
            validators.push(Validators.required);
        }
        if (options.min) {
            validators.push(Validators.min(options.min));
        }
        if (options.minLength) {
            validators.push(Validators.minLength(options.minLength));
        }
        if (options.max) {
            validators.push(Validators.max(options.max));
        }
        if (options.maxLength) {
            validators.push(Validators.maxLength(options.maxLength));
        }
        if (options.email) {
            validators.push(Validators.email);
        }
        if (options.pattern) {
            validators.push(Validators.pattern(options.pattern));
        }
        validators.push(...this.createCustomValidators('sync', options));
        return validators;
    }
    createAsyncValidators(options) {
        return this.createCustomValidators('async', options);
    }
    createCustomValidators(type, options) {
        const validatorIds = toArray(type === 'sync' ? options === null || options === void 0 ? void 0 : options.custom : options === null || options === void 0 ? void 0 : options.customAsync);
        const validators = validatorIds.map(id => new ValidatorDefinition(id))
            .filter(def => {
            if (this.registry.hasValidator(def.id, type)) {
                return true;
            }
            else {
                console.warn(`No ${type} validator registered with id [${def.id}].`);
                return false;
            }
        })
            .map(def => this.registry.getValidator(def, type));
        if (type === 'sync') {
            validators.push(...toArray(options === null || options === void 0 ? void 0 : options.customFn));
        }
        else {
            validators.push(...toArray(options === null || options === void 0 ? void 0 : options.customAsyncFn));
        }
        return validators;
    }
}
ValidatorFactoryService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: ValidatorFactoryService, deps: [{ token: i0.Injector }, { token: VALIDATORS, optional: true }], target: i0.ɵɵFactoryTarget.Injectable });
ValidatorFactoryService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: ValidatorFactoryService, providedIn: 'any' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: ValidatorFactoryService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'any'
                }]
        }], ctorParameters: function () {
        return [{ type: i0.Injector }, { type: Array, decorators: [{
                        type: Optional
                    }, {
                        type: Inject,
                        args: [VALIDATORS]
                    }] }];
    } });

function flattenArray(itemsList) {
    const flattenedArray = [];
    for (const itemList of itemsList !== null && itemsList !== void 0 ? itemsList : []) {
        for (const item of itemList) {
            flattenedArray.push(item);
        }
    }
    return flattenedArray;
}

class AbstractRegistry {
    constructor(items) {
        this.items = {};
        const registeredIds = new Set();
        for (const item of flattenArray(items)) {
            this.validate(item);
            this.ids(item).forEach(id => {
                if (registeredIds.has(id)) {
                    throw new Error(`Id [${id}] already exist.`);
                }
                registeredIds.add(id);
                this.items[id] = item;
            });
        }
    }
    hasItem(id) {
        return id in this.items;
    }
    getItem(id) {
        if (id in this.items) {
            return this.items[id];
        }
        else {
            throw new Error(`No item with id [${id}] found in registry.`);
        }
    }
}

const AFF_CONTROL_COMPONENTS = new InjectionToken('AFF_CONTROL_COMPONENTS');

class ControlRegistry extends AbstractRegistry {
    constructor(controlComponentsPerModule) {
        super(controlComponentsPerModule);
    }
    ids(item) {
        return item[META_COMPONENT_OPTIONS_KEY].type.split(',');
    }
    validate(control) {
        if (!control[META_COMPONENT_OPTIONS_KEY]) {
            throw new Error(`Control component must be decorated with [@Control] decorator.`);
        }
    }
    hasControlFactory(type) {
        if (this.hasItem(type)) {
            const metaData = this.getComponentMetaData(type);
            return metaData.controlFactory !== undefined;
        }
        else {
            return false;
        }
    }
    getControlFactory(type) {
        const def = this.getDefinition(type);
        if (def.controlFactory !== undefined) {
            return def.controlFactory;
        }
        return null;
    }
    getDefinition(type) {
        const component = this.getItem(type);
        const componentMetaData = component[META_COMPONENT_OPTIONS_KEY];
        return {
            type: type,
            inline: componentMetaData.inline,
            controlFactory: componentMetaData.controlFactory,
            component: component,
            internalType: componentMetaData.internalType
        };
    }
    getComponentMetaData(type) {
        return this.getItem(type)[META_COMPONENT_OPTIONS_KEY];
    }
}
ControlRegistry.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: ControlRegistry, deps: [{ token: AFF_CONTROL_COMPONENTS, optional: true }], target: i0.ɵɵFactoryTarget.Injectable });
ControlRegistry.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: ControlRegistry, providedIn: 'any' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: ControlRegistry, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'any'
                }]
        }], ctorParameters: function () {
        return [{ type: Array, decorators: [{
                        type: Optional
                    }, {
                        type: Inject,
                        args: [AFF_CONTROL_COMPONENTS]
                    }] }];
    } });

//TODO: Check if introducing circ dependency with FastFormBuilder is best solution
//Possible solution: Add static control enhancement method to add wrapper to control
class ControlFactoryV2 {
    constructor(validatorFactory, cr) {
        this.validatorFactory = validatorFactory;
        this.cr = cr;
    }
    create(wrapper) {
        switch (wrapper.controlType) {
            case 'control':
                return this.dynamicControl(wrapper.initialState, wrapper.question);
            case 'group':
                return this.group(wrapper.question, this.createSubGroupControls(wrapper.groupQuestion));
            case 'array':
                return this.array(wrapper.question, this.create(wrapper.arrayQuestion));
        }
    }
    control(state, question) {
        return this.dynamicControl(state, question);
    }
    dynamicControl(state, question) {
        let control;
        const controlFactory = this.cr.getControlFactory(question.type);
        if (controlFactory) {
            control = this.createViaFactoryMethod(controlFactory, question, state);
        }
        else {
            control = new FormControl(state, Object.assign(Object.assign({}, this.createValidators(question)), { updateOn: question.updateOn, nonNullable: question.nonNullable }));
        }
        control[ControlWrapperKey] = ControlWrapperV2.fromControl(state, question);
        return control;
    }
    group(question, groupControls) {
        const controlFactory = this.cr.getControlFactory(question.type);
        let group;
        if (controlFactory) {
            group = this.createViaFactoryMethod(controlFactory, question);
        }
        else {
            group = new FormGroup(groupControls !== null && groupControls !== void 0 ? groupControls : {}, Object.assign(Object.assign({}, this.createValidators(question)), { updateOn: question.updateOn }));
        }
        const groupQuestions = {};
        Object.keys(groupControls !== null && groupControls !== void 0 ? groupControls : {}).forEach(key => {
            const childControl = this.deriveDefinition(null, (groupControls !== null && groupControls !== void 0 ? groupControls : {})[key]);
            if (!childControl.question.id) {
                childControl.question.id = key;
            }
            groupQuestions[key] = childControl;
        });
        group[ControlWrapperKey] = ControlWrapperV2.fromGroup(question, groupQuestions);
        return group;
    }
    array(question, arrayQuestion) {
        const controlFactory = this.cr.getControlFactory(question.type);
        let array;
        if (controlFactory) {
            array = this.createViaFactoryMethod(controlFactory, question);
        }
        else {
            array = new FormArray([], Object.assign(Object.assign({}, this.createValidators(question)), { updateOn: question.updateOn }));
        }
        if (arrayQuestion) {
            array[ControlWrapperKey] = ControlWrapperV2.fromArray(question, this.deriveDefinition(null, arrayQuestion));
        }
        else {
            array[ControlWrapperKey] = ControlWrapperV2.fromArray(question);
        }
        return array;
    }
    deriveDefinition(key, control) {
        if (!hasControlWrapper(control)) {
            throw new Error(`Cannot create control which is not created via the [${FastFormBuilder.name}].`);
        }
        const wrapper = control[ControlWrapperKey];
        if (wrapper.controlType === 'control') {
            return wrapper;
        }
        else if (wrapper.controlType === 'group') {
            return wrapper;
        }
        else if (wrapper.controlType === 'array') {
            return wrapper;
        }
        else {
            throw new Error(`The control type [${wrapper.controlType}]} is not supported.`);
        }
    }
    createValidators(question) {
        return {
            validators: this.createValidatorsByType('sync', question.validators, question.validation),
            asyncValidators: this.createValidatorsByType('async', question.asyncValidators, question.validation)
        };
    }
    createValidatorsByType(type, validators, opts) {
        const combinedValidators = [];
        if (validators) {
            if (validators instanceof Array) {
                validators.forEach(v => combinedValidators.push(v));
            }
            else {
                combinedValidators.push(validators);
            }
        }
        if (opts) {
            if (type === 'sync') {
                combinedValidators.push(...this.validatorFactory.createValidators(opts));
            }
            else if (type === 'async') {
                combinedValidators.push(...this.validatorFactory.createAsyncValidators(opts));
            }
        }
        if (combinedValidators.length === 0) {
            return null;
        }
        else {
            return combinedValidators;
        }
    }
    createSubGroupControls(groupQuestions) {
        const groupDef = {};
        Object.keys(groupQuestions).forEach(key => {
            groupDef[key] = this.create(groupQuestions[key]);
        });
        return groupDef;
    }
    createViaFactoryMethod(method, question, defaultValue) {
        return method(Object.assign(Object.assign({}, question), { defaultValue: defaultValue }), Object.assign(Object.assign({}, this.createValidators(question)), { fb: new FastFormBuilder(this) }));
    }
}
ControlFactoryV2.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: ControlFactoryV2, deps: [{ token: ValidatorFactoryService }, { token: ControlRegistry }], target: i0.ɵɵFactoryTarget.Injectable });
ControlFactoryV2.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: ControlFactoryV2, providedIn: 'any' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: ControlFactoryV2, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'any'
                }]
        }], ctorParameters: function () { return [{ type: ValidatorFactoryService }, { type: ControlRegistry }]; } });

class FastFormBuilder {
    constructor(cf) {
        this.cf = cf;
    }
    control(state, opts) {
        return this.dynamicControl(state, opts);
    }
    controlGroup(state, opts) {
        return this.dynamicControl(state, opts);
    }
    dynamicControl(state, opts) {
        return this.cf.dynamicControl(state, opts);
    }
    /**
     * Experimental
     */
    group(question, groupControls) {
        var _a;
        return this.cf.group(Object.assign(Object.assign({}, question), { type: (_a = question.type) !== null && _a !== void 0 ? _a : 'group-v2' }), groupControls);
    }
    /**
     * Experimental
     */
    array(question, arrayQuestion) {
        var _a;
        return this.cf.array(Object.assign(Object.assign({}, question), { type: (_a = question.type) !== null && _a !== void 0 ? _a : 'array-v2' }), arrayQuestion);
    }
    /**
     * Experimental
     */
    newArrayEntry(array) {
        if (hasControlWrapper(array)) {
            const wrapper = array[ControlWrapperKey];
            if (wrapper.controlType !== 'array') {
                throw new Error(`Cannot create array entry for component type [${wrapper.controlType}].`);
            }
            return this.cf.create(wrapper.arrayQuestion);
        }
        else {
            throw new Error(`Cannot create array entry when control is not created via [${FastFormBuilder.name}].`);
        }
    }
}
FastFormBuilder.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: FastFormBuilder, deps: [{ token: ControlFactoryV2 }], target: i0.ɵɵFactoryTarget.Injectable });
FastFormBuilder.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: FastFormBuilder, providedIn: 'any' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: FastFormBuilder, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'any'
                }]
        }], ctorParameters: function () { return [{ type: ControlFactoryV2 }]; } });
const ControlWrapperKey = 'aff_wrapper';
function hasControlWrapper(control) {
    return ControlWrapperKey in control;
}

const CONTROL_PROPERTIES = new InjectionToken('AFF_CONTROL_PROPERTIES');
const FORM_CONTROL = new InjectionToken('AFF_FORM_CONTROL');
const CONTROL_ID = new InjectionToken('AFF_CONTROL_ID');

/**
 * @deprecated Will be removed in version [3.0.0]. Use dependency
 * injection pattern instead.
 */
class BaseFormInlineComponent {
}

class ControlIdImpl {
    constructor(parentParts = []) {
        this.parentParts = parentParts;
        this.parts = parentParts.map((part) => ({
            id: part.id,
            indexProvider: part.indexProvider,
        }));
    }
    addPart(part) {
        return new ControlIdImpl([
            ...this.parts,
            {
                id: part,
            },
        ]);
    }
    addIndex(part, indexProvider) {
        let clonedId = new ControlIdImpl(this.parts);
        if (clonedId.parts.length > 0) {
            const lastPart = clonedId.parts[clonedId.parts.length - 1];
            if (lastPart.indexProvider === undefined) {
                lastPart.indexProvider = indexProvider;
            }
            else {
                clonedId = new ControlIdImpl([...clonedId.parts, { indexProvider: indexProvider }]);
            }
        }
        else if (clonedId.parts.length === 0) {
            if (part === undefined) {
                clonedId = new ControlIdImpl([{ indexProvider: indexProvider }]);
            }
            else {
                clonedId = new ControlIdImpl([{ indexProvider: indexProvider }, { id: part }]);
            }
        }
        return clonedId;
    }
    getId() {
        return this.parts
            .map((part) => {
            if (part.id !== undefined && part.indexProvider !== undefined) {
                return `${part.id}[${part.indexProvider.index}]`;
            }
            else if (part.indexProvider !== undefined) {
                return `[${part.indexProvider.index}]`;
            }
            else {
                return part.id;
            }
        })
            .join('.');
    }
}

class QuestionDefinition {
    constructor(question) {
        this.id = question.id;
        this.label = question.label;
        this.hidden = question.hidden;
        this.disabled = question.disabled;
        this.validation = question.validation;
        this.properties = question.properties;
        this.defaultValue = question.defaultValue;
        this.children = question.children;
    }
}

class FormRenderService {
    constructor(controlRegistry, injector) {
        this.controlRegistry = controlRegistry;
        this.injector = injector;
    }
    renderOnly(viewContainerRef, control, opts) {
        if (!hasControlWrapper(control)) {
            throw new Error('Cannot render control which has no control wrapper.');
        }
        // console.log(parent);
        const question = control[ControlWrapperKey].question;
        // console.log(question);
        // console.log((parent[QuestionWrapper] as ControlWrapperV2).question);
        const def = this.controlRegistry.getDefinition(question.type);
        const providers = [
            { provide: FORM_CONTROL, useValue: control },
            {
                provide: QuestionDefinition, useValue: new QuestionDefinition(Object.assign({ id: '' }, question))
            }
        ];
        //TODO: Check if this injection is valid
        if (question.properties) {
            providers.push({ provide: CONTROL_PROPERTIES, useValue: question.properties });
        }
        const controlComponentRef = viewContainerRef.createComponent(def.component, {
            injector: Injector.create({
                providers,
                parent: opts.injector ? opts.injector : this.injector
            })
        });
        // this.renderer.appendChild(viewContainerRef.element.nativeElement, ComponentRef);
        if (control instanceof FormGroup) {
            // Object.keys(parent.controls).forEach(key => {
            //   const controlComponentRef = viewContainerRef.createComponent(def.component, {
            //     injector: Injector.create({
            //       providers: [
            //         {provide: FORM_CONTROL, useValue: parent}
            //       ]
            //       // parent: injector ? injector : this.injector
            //     })
            //   });
            //
            // })
        }
        // if (this.shouldInitialize(controlComponentRef.instance)) {
        //   this.initializeComponent(parent, question, formDefinition.component.name, controlComponentRef.instance);
        // }
        return controlComponentRef;
    }
    render(viewContainerRef, parent, question, formDefinition, injector, actionService, indexDirective) {
        const controlComponentRef = viewContainerRef.createComponent(formDefinition.component, {
            injector: Injector.create({
                providers: this.createProviders(question, parent, injector, actionService, indexDirective),
                parent: injector ? injector : this.injector
            })
        });
        if (this.shouldInitialize(controlComponentRef.instance)) {
            this.initializeComponent(parent, question, formDefinition.component.name, controlComponentRef.instance);
        }
        return controlComponentRef;
    }
    createProviders(question, parent, injector, actionService, indexDirective) {
        var _a;
        const id = injector.get(CONTROL_ID, new ControlIdImpl());
        let control = null;
        if (this.controlRegistry.hasItem(question.type)) {
            const def = this.controlRegistry.getDefinition(question.type);
            if ((def.internalType === 'control' && parent instanceof FormControl)
                || def.inline) {
                control = parent;
            }
            else {
                control = parent.get(question.id);
            }
        }
        return [
            { provide: QuestionDefinition, useValue: new QuestionDefinition(question) },
            { provide: CONTROL_PROPERTIES, useValue: (_a = question.properties) !== null && _a !== void 0 ? _a : {} },
            { provide: CONTROL_ID, useValue: this.createControlId(id, question.id, parent, indexDirective) },
            { provide: FORM_CONTROL, useValue: control },
            { provide: ActionService, useValue: actionService }
        ];
    }
    createControlId(id, questionId, control, indexDirective) {
        if (indexDirective && (control instanceof FastFormGroup || control instanceof FastFormControl)) {
            return id.addIndex(questionId, control);
        }
        else {
            return id.addPart(questionId);
        }
    }
    shouldInitialize(component) {
        return (component instanceof BaseFormArrayComponent ||
            component instanceof BaseFormInlineComponent ||
            component instanceof BaseFormControlComponent ||
            component instanceof BaseFormGroupComponent);
    }
    // TODO better type check
    initializeComponent(control, question, componentName, component) {
        if (component instanceof BaseFormArrayComponent && control instanceof FormGroup) {
            this.initializeFormArrayComponent(control, question, component);
        }
        else if (component instanceof BaseFormInlineComponent && control instanceof FormGroup) {
            this.initializeFormInlineComponent(control, question, component);
        }
        else if (component instanceof BaseFormControlComponent &&
            (control instanceof FormGroup || control instanceof FormControl)) {
            this.initializeFormControlComponent(control, question, component);
        }
        else if (component instanceof BaseFormGroupComponent && control instanceof FormGroup) {
            this.initializeFormGroupComponent(control, question, component);
        }
        else {
            throw new Error(`Cannot create component [${componentName}] for question with id [${question.id}]`);
        }
    }
    initializeFormArrayComponent(formGroup, question, component) {
        var _a, _b;
        component.formArray = formGroup.controls[question.id];
        const formChildren = (_a = question.children) !== null && _a !== void 0 ? _a : [];
        if (formChildren.length > 1) {
            throw new Error('Only one children is allowed in a form array.');
        }
        if (formChildren.length === 0) {
            console.warn(`No question registered in form array [${question.id}].`);
        }
        else {
            component.question = formChildren[0];
        }
        component.properties = (_b = question === null || question === void 0 ? void 0 : question.properties) !== null && _b !== void 0 ? _b : {};
    }
    initializeFormInlineComponent(formGroup, question, component) {
        var _a;
        component.formGroup = formGroup;
        component.questions = (_a = question.children) !== null && _a !== void 0 ? _a : [];
    }
    initializeFormGroupComponent(formGroup, question, component) {
        var _a, _b;
        component.formGroup = formGroup.controls[question.id];
        component.questions = (_a = question.children) !== null && _a !== void 0 ? _a : [];
        component.properties = (_b = question === null || question === void 0 ? void 0 : question.properties) !== null && _b !== void 0 ? _b : {};
    }
    initializeFormControlComponent(control, question, component) {
        var _a;
        if (control instanceof FormGroup) {
            component.formGroup = control;
            component.control = control.controls[question.id];
        }
        else {
            component.control = control;
        }
        component.question = question;
        component.properties = (_a = question === null || question === void 0 ? void 0 : question.properties) !== null && _a !== void 0 ? _a : {};
    }
}
FormRenderService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: FormRenderService, deps: [{ token: ControlRegistry }, { token: i0.Injector }], target: i0.ɵɵFactoryTarget.Injectable });
FormRenderService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: FormRenderService, providedIn: 'any' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: FormRenderService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'any'
                }]
        }], ctorParameters: function () { return [{ type: ControlRegistry }, { type: i0.Injector }]; } });

class FormRendererDirective {
    constructor(viewContainerRef, renderer, renderService, injector, groupDirective) {
        this.viewContainerRef = viewContainerRef;
        this.renderer = renderer;
        this.renderService = renderService;
        this.injector = injector;
        this.groupDirective = groupDirective;
        this.renderControl = null;
        this.renderControlName = null;
    }
    ngOnInit() {
        const control = this.controlToRender;
        if (control) {
            this.render(control);
        }
    }
    get controlToRender() {
        var _a, _b, _c;
        let control = null;
        if (this.renderControlName) {
            control = (_b = (_a = this.groupDirective) === null || _a === void 0 ? void 0 : _a.form.controls[this.renderControlName]) !== null && _b !== void 0 ? _b : null;
        }
        else if (this.renderControl) {
            control = this.renderControl;
        }
        else if ((_c = this.groupDirective) === null || _c === void 0 ? void 0 : _c.control) {
            control = this.groupDirective.control;
        }
        return control;
    }
    render(control) {
        const isHtmlElement = this.viewContainerRef.element.nativeElement instanceof HTMLElement;
        const componentRef = this.renderService.renderOnly(this.viewContainerRef, control, { injector: this.injector });
        if (isHtmlElement) {
            this.renderer.appendChild(this.viewContainerRef.element.nativeElement, componentRef.location.nativeElement);
        }
    }
}
FormRendererDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: FormRendererDirective, deps: [{ token: i0.ViewContainerRef }, { token: i0.Renderer2 }, { token: FormRenderService }, { token: i0.Injector }, { token: i2.FormGroupDirective, optional: true }], target: i0.ɵɵFactoryTarget.Directive });
FormRendererDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "15.2.10", type: FormRendererDirective, selector: "[affRenderer],[renderControl],[renderControlName]", inputs: { renderControl: "renderControl", renderControlName: "renderControlName" }, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: FormRendererDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[affRenderer],[renderControl],[renderControlName]'
                }]
        }], ctorParameters: function () {
        return [{ type: i0.ViewContainerRef }, { type: i0.Renderer2 }, { type: FormRenderService }, { type: i0.Injector }, { type: i2.FormGroupDirective, decorators: [{
                        type: Optional
                    }] }];
    }, propDecorators: { renderControl: [{
                type: Input
            }], renderControlName: [{
                type: Input
            }] } });

let FastFormArrayV2Component = class FastFormArrayV2Component {
    constructor(array, fb) {
        this.array = array;
        this.fb = fb;
    }
    addRow(i) {
        if (this.array && hasControlWrapper(this.array)) {
            if (i !== undefined) {
                this.array.insert(i + 1, this.fb.newArrayEntry(this.array));
            }
            else {
                this.array.push(this.fb.newArrayEntry(this.array));
            }
        }
    }
    deleteRow(i) {
        var _a;
        if (i !== undefined) {
            (_a = this.array) === null || _a === void 0 ? void 0 : _a.removeAt(i);
        }
    }
};
FastFormArrayV2Component.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: FastFormArrayV2Component, deps: [{ token: FORM_CONTROL }, { token: FastFormBuilder }], target: i0.ɵɵFactoryTarget.Component });
FastFormArrayV2Component.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "15.2.10", type: FastFormArrayV2Component, selector: "aff-form-array-v2", ngImport: i0, template: "<div *ngFor=\"let subForm of array.controls; let i = index\" class=\"aff-array-entry\">\n    <div class=\"array-container\">\n        <div [renderControl]=\"subForm\"></div>\n    </div>\n    <div class=\"array-actions\">\n        <button (click)=\"addRow(i)\">Add</button>\n        <button (click)=\"deleteRow(i)\">Delete</button>\n    </div>\n</div>\n<button (click)=\"addRow()\" *ngIf=\"array.controls.length === 0\">\n    <span>New entry</span>\n</button>\n", dependencies: [{ kind: "directive", type: i2$1.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { kind: "directive", type: i2$1.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "directive", type: FormRendererDirective, selector: "[affRenderer],[renderControl],[renderControlName]", inputs: ["renderControl", "renderControlName"] }] });
FastFormArrayV2Component = __decorate([
    Control({
        type: 'array-v2',
        controlType: 'array'
    }),
    __metadata("design:paramtypes", [FormArray,
        FastFormBuilder])
], FastFormArrayV2Component);
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: FastFormArrayV2Component, decorators: [{
            type: Component,
            args: [{ selector: 'aff-form-array-v2', template: "<div *ngFor=\"let subForm of array.controls; let i = index\" class=\"aff-array-entry\">\n    <div class=\"array-container\">\n        <div [renderControl]=\"subForm\"></div>\n    </div>\n    <div class=\"array-actions\">\n        <button (click)=\"addRow(i)\">Add</button>\n        <button (click)=\"deleteRow(i)\">Delete</button>\n    </div>\n</div>\n<button (click)=\"addRow()\" *ngIf=\"array.controls.length === 0\">\n    <span>New entry</span>\n</button>\n" }]
        }], ctorParameters: function () {
        return [{ type: i2.FormArray, decorators: [{
                        type: Inject,
                        args: [FORM_CONTROL]
                    }] }, { type: FastFormBuilder }];
    } });

let FastFormGroupV2Component = class FastFormGroupV2Component {
    constructor(formGroup, def) {
        var _a, _b;
        this.formGroup = formGroup;
        this.props = Object.assign(Object.assign({}, def.properties), { alignment: (_b = (_a = def.properties) === null || _a === void 0 ? void 0 : _a.alignment) !== null && _b !== void 0 ? _b : 'row' });
    }
    get controlKeys() {
        var _a;
        return Object.keys((_a = this.formGroup.controls) !== null && _a !== void 0 ? _a : {});
    }
};
FastFormGroupV2Component.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: FastFormGroupV2Component, deps: [{ token: FORM_CONTROL }, { token: QuestionDefinition }], target: i0.ɵɵFactoryTarget.Component });
FastFormGroupV2Component.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "15.2.10", type: FastFormGroupV2Component, selector: "aff-form-group-v2", ngImport: i0, template: "<div [ngClass]=\"{\n  'row': props.alignment === 'row',\n  'column': props.alignment === 'column'\n}\" class=\"aff-form-group-v2\">\n  <ng-container *ngFor=\"let key of controlKeys\" [renderControl]=\"formGroup.get(key)\"></ng-container>\n</div>\n", dependencies: [{ kind: "directive", type: i2$1.NgClass, selector: "[ngClass]", inputs: ["class", "ngClass"] }, { kind: "directive", type: i2$1.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { kind: "directive", type: FormRendererDirective, selector: "[affRenderer],[renderControl],[renderControlName]", inputs: ["renderControl", "renderControlName"] }] });
FastFormGroupV2Component = __decorate([
    Control({
        type: 'group-v2',
        controlType: 'group'
    }),
    __metadata("design:paramtypes", [FormGroup,
        QuestionDefinition])
], FastFormGroupV2Component);
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: FastFormGroupV2Component, decorators: [{
            type: Component,
            args: [{ selector: 'aff-form-group-v2', template: "<div [ngClass]=\"{\n  'row': props.alignment === 'row',\n  'column': props.alignment === 'column'\n}\" class=\"aff-form-group-v2\">\n  <ng-container *ngFor=\"let key of controlKeys\" [renderControl]=\"formGroup.get(key)\"></ng-container>\n</div>\n" }]
        }], ctorParameters: function () {
        return [{ type: i2.FormGroup, decorators: [{
                        type: Inject,
                        args: [FORM_CONTROL]
                    }] }, { type: QuestionDefinition }];
    } });

class FastFormArray extends FormArray {
    constructor(question, controlFactory) {
        super([]);
        this.question = question;
        this.controlFactory = controlFactory;
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    setValue(values, options) {
        if (values instanceof Array) {
            this.updateControlCount(values.length, { emitEvent: false });
        }
        else {
            // TODO error handling
        }
        super.setValue(values, options);
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    patchValue(values, options) {
        if (values instanceof Array) {
            this.updateControlCount(values.length, { emitEvent: false });
        }
        else {
            // TODO error handling
        }
        super.patchValue(values, options);
    }
    addRow(index, options) {
        this.addControlsToArray(1, {
            index: index,
            emitEvent: options === null || options === void 0 ? void 0 : options.emitEvent
        });
    }
    copyRow(index, options) {
        const listValue = this.getRawValue();
        listValue.splice(index, 0, listValue[index]);
        const newIndex = (options === null || options === void 0 ? void 0 : options.insertAfter) != null ? (options === null || options === void 0 ? void 0 : options.insertAfter) + 1 : index + 1;
        this.addControlsToArray(1, {
            index: newIndex,
            emitEvent: false
        });
        this.controls[newIndex].patchValue(listValue[index], options);
    }
    updateControlCount(dataLength, options) {
        if (dataLength > this.controls.length) {
            this.addControlsToArray(dataLength - this.controls.length, options);
        }
        if (dataLength < this.controls.length) {
            this.removeControlsFromArray(this.controls.length - dataLength, options);
        }
    }
    addControlsToArray(amount, options) {
        for (let i = 0; i < amount; i++) {
            this.controlFactory.createFromQuestion(this, this.question, options);
        }
    }
    removeControlsFromArray(amount, options) {
        const lastControlIndex = this.controls.length - 1;
        const startIndex = (options === null || options === void 0 ? void 0 : options.index) !== undefined ? options.index : lastControlIndex;
        for (let i = startIndex; i > startIndex - amount; i--) {
            this.removeAt(i, { emitEvent: options === null || options === void 0 ? void 0 : options.emitEvent });
        }
    }
}

class ControlWrapper {
    constructor(id, control, controlType) {
        this.id = id;
        if (controlType === 'action') {
            this.control = null;
            this.action = control;
        }
        else if (control instanceof AbstractControl) {
            this.control = control;
            this.action = null;
        }
        else {
            throw new Error('Unsupported type.');
        }
    }
    static forFormControl(id, formControl) {
        return new ControlWrapper(id, formControl, 'control');
    }
    static forFormArray(id, formArray) {
        return new ControlWrapper(id, formArray, 'array');
    }
    static forAction(id, actionControl) {
        return new ControlWrapper(id, actionControl, 'action');
    }
    addToParent(parent, options) {
        if (parent instanceof FormGroup) {
            this.addToGroup(parent, {
                emitEvent: options === null || options === void 0 ? void 0 : options.emitEvent
            });
        }
        else {
            this.addToArray(parent, options === null || options === void 0 ? void 0 : options.index, {
                emitEvent: options === null || options === void 0 ? void 0 : options.emitEvent
            });
        }
    }
    addToGroup(parent, options) {
        if (this.control !== null) {
            parent.addControl(this.id, this.control, options);
        }
        else if (this.action !== null) {
            if (parent instanceof FastFormGroup) {
                parent.addControl(this.id, this.action, options);
            }
            else {
                console.warn('Cannot add action to standard reactive form.');
            }
        }
        else {
            throw new Error(`Cannot add control or action to parent.`);
        }
    }
    addToArray(parent, index, options) {
        if (this.control !== null) {
            if (index !== undefined) {
                parent.insert(index, this.control, options);
            }
            else {
                parent.push(this.control, options);
            }
        }
        else if (this.action !== null) {
            console.warn('Actions are not supported in form arrays.');
        }
        else {
            throw new Error(`Cannot add control or action to parent.`);
        }
    }
}

class ControlFactoryService {
    constructor(validatorFactory, controlRegistry, fb) {
        this.validatorFactory = validatorFactory;
        this.controlRegistry = controlRegistry;
        this.fb = fb;
    }
    createFromQuestions(parent, questions, options) {
        for (const question of questions || []) {
            this.createFromQuestion(parent, question, options);
        }
    }
    createFromQuestion(parent, question, options) {
        var _a;
        let createdControls;
        if (this.controlRegistry.hasItem(question.type)) {
            const def = this.controlRegistry.getDefinition(question.type);
            if (def.inline) {
                createdControls = flattenArray(((_a = question.children) !== null && _a !== void 0 ? _a : []).map((childQuestion) => {
                    return this.createControl(childQuestion);
                }));
            }
            else {
                createdControls = this.createControl(question);
            }
        }
        else {
            createdControls = this.createControl(question);
        }
        createdControls.map((control) => control.addToParent(parent, options));
    }
    createFormControl(question) {
        var _a;
        return (_a = this.createControlFromDecoratedComponents(question)) !== null && _a !== void 0 ? _a : this.createControlDefault(question);
    }
    createControl(question) {
        var _a;
        const wrappers = [];
        if (this.controlRegistry.hasItem(question.type)) {
            const definition = this.controlRegistry.getDefinition(question.type);
            if (definition.internalType === 'control') {
                wrappers.push(ControlWrapper.forFormControl(question.id, this.createAndInitFormControl(question)));
            }
            else if (definition.internalType === 'action') {
                wrappers.push(ControlWrapper.forAction(question.id, this.createAndInitFormAction(question)));
            }
            else if (definition.internalType === 'array') {
                // TODO length check and assertions, create group automatically if more than one
                wrappers.push(ControlWrapper.forFormArray(question.id, new FastFormArray(((_a = question.children) !== null && _a !== void 0 ? _a : [])[0], this)));
            }
            else if (definition.internalType === 'group') {
                wrappers.push(ControlWrapper.forFormControl(question.id, this.createAndInitFormGroup(question)));
            }
        }
        if (wrappers.length === 0) {
            console.error(`Form control with type [${question.type}] not found.`);
        }
        return wrappers;
    }
    createAndInitFormControl(question) {
        const control = this.createFormControl(question);
        const validator = this.validatorFactory.createValidators(question.validation);
        const asyncValidator = this.validatorFactory.createAsyncValidators(question.validation);
        control.setValidators(validator);
        control.setAsyncValidators(asyncValidator);
        if (question.disabled) {
            control.disable({ emitEvent: false });
        }
        return control;
    }
    createControlFromDecoratedComponents(question) {
        if (this.controlRegistry.hasControlFactory(question.type)) {
            const def = this.controlRegistry.getDefinition(question.type);
            if (def.controlFactory !== undefined) {
                return def.controlFactory(question, {
                    fb: this.fb,
                    validators: this.validatorFactory.createValidators(question.validation),
                    asyncValidators: this.validatorFactory.createAsyncValidators(question.validation),
                });
            }
        }
        return undefined;
    }
    createControlDefault(question) {
        return new FastFormControl(question, question.defaultValue);
    }
    createAndInitFormAction(question) {
        const definition = this.controlRegistry.getDefinition(question.type);
        if (definition) {
            if (definition.controlFactory) {
                return definition.controlFactory(question, {
                    fb: this.fb,
                    validators: this.validatorFactory.createValidators(question.validation),
                    asyncValidators: this.validatorFactory.createAsyncValidators(question.validation),
                });
            }
        }
        return new FromActionControlInternal();
    }
    createAndInitFormGroup(question) {
        var _a;
        const subFormGroup = new FastFormGroup((_a = question.children) !== null && _a !== void 0 ? _a : [], this);
        if (question.disabled) {
            subFormGroup.disable({ emitEvent: false });
        }
        return subFormGroup;
    }
}
ControlFactoryService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: ControlFactoryService, deps: [{ token: ValidatorFactoryService }, { token: ControlRegistry }, { token: FastFormBuilder }], target: i0.ɵɵFactoryTarget.Injectable });
ControlFactoryService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: ControlFactoryService, providedIn: 'any' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: ControlFactoryService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'any',
                }]
        }], ctorParameters: function () { return [{ type: ValidatorFactoryService }, { type: ControlRegistry }, { type: FastFormBuilder }]; } });

let FastFormGroupComponent = class FastFormGroupComponent {
    set formGroup(formGroup) {
        this._formGroup = formGroup;
    }
    constructor(controlFactory, validatorFactory, formRenderService, injector, controlRegistry, actionService, indexDirective, http) {
        this.controlFactory = controlFactory;
        this.validatorFactory = validatorFactory;
        this.formRenderService = formRenderService;
        this.injector = injector;
        this.controlRegistry = controlRegistry;
        this.indexDirective = indexDirective;
        this.http = http;
        this.action = new EventEmitter();
        this.submitEvent = new EventEmitter();
        this._formGroup = new FastFormGroup([], this.controlFactory);
        if (actionService) {
            this._actionService = actionService;
        }
        else {
            this._actionService = new ActionService();
        }
    }
    ngOnInit() {
        this._formGroup.questionChanges.subscribe(() => {
            this.render();
        });
        this._actionsSub = this._actionService.actions.subscribe({
            next: event => {
                this.action.emit(event);
            }
        });
    }
    ngOnChanges() {
        this.render();
    }
    ngOnDestroy() {
        this._actionsSub.unsubscribe();
    }
    processOnSubmit(event) {
        this._formGroup.markAllAsTouched();
        if (this._formGroup.valid) {
            this.submitEvent.next({
                event: event,
                data: this._formGroup.value
            });
        }
    }
    render() {
        this.componentViewContainerRef.clear();
        this._formGroup.questions.filter(question => !question.hidden)
            .forEach(question => {
            this.createComponent(question);
        });
    }
    createComponent(question) {
        if (this.controlRegistry.hasItem(question.type)) {
            const controlDefinition = this.controlRegistry.getDefinition(question.type);
            this.formRenderService.render(this.componentViewContainerRef, this._formGroup, question, controlDefinition, this.injector, this._actionService, this.indexDirective);
        }
    }
};
FastFormGroupComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: FastFormGroupComponent, deps: [{ token: ControlFactoryService }, { token: ValidatorFactoryService }, { token: FormRenderService }, { token: i0.Injector }, { token: ControlRegistry }, { token: ActionService, optional: true }, { token: ArrayIndexDirective, optional: true }, { token: i7.HttpClient, optional: true }], target: i0.ɵɵFactoryTarget.Component });
FastFormGroupComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "15.2.10", type: FastFormGroupComponent, selector: "aff-form-group", inputs: { endpoint: "endpoint", formGroup: "formGroup" }, outputs: { action: "action", submitEvent: "submitEvent" }, viewQueries: [{ propertyName: "componentViewContainerRef", first: true, predicate: ["componentViewContainer"], descendants: true, read: ViewContainerRef, static: true }], usesOnChanges: true, ngImport: i0, template: "<form (ngSubmit)=\"processOnSubmit($event)\" [formGroup]=\"_formGroup\" class=\"aff-form-group\">\n  <ng-container #componentViewContainer></ng-container>\n\n  <ng-container *ngFor=\"let question of _formGroup.questions\">\n    <ng-container *ngIf=\"question.type === 'group'\">\n      <aff-form-group [formGroup]=\"$any(_formGroup.get(question.id))\"></aff-form-group>\n    </ng-container>\n  </ng-container>\n\n  <button style=\"display: none\" type=\"submit\">hidden submit</button>\n</form>\n", dependencies: [{ kind: "directive", type: i2$1.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { kind: "directive", type: i2$1.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "directive", type: i2.ɵNgNoValidate, selector: "form:not([ngNoForm]):not([ngNativeValidate])" }, { kind: "directive", type: i2.NgControlStatusGroup, selector: "[formGroupName],[formArrayName],[ngModelGroup],[formGroup],form:not([ngNoForm]),[ngForm]" }, { kind: "directive", type: i2.FormGroupDirective, selector: "[formGroup]", inputs: ["formGroup"], outputs: ["ngSubmit"], exportAs: ["ngForm"] }, { kind: "component", type: FastFormGroupComponent, selector: "aff-form-group", inputs: ["endpoint", "formGroup"], outputs: ["action", "submitEvent"] }] });
FastFormGroupComponent = __decorate([
    Control({
        type: 'group',
        controlType: 'group'
    }),
    __metadata("design:paramtypes", [ControlFactoryService,
        ValidatorFactoryService,
        FormRenderService,
        Injector,
        ControlRegistry,
        ActionService,
        ArrayIndexDirective,
        HttpClient])
], FastFormGroupComponent);
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: FastFormGroupComponent, decorators: [{
            type: Component,
            args: [{ selector: 'aff-form-group', template: "<form (ngSubmit)=\"processOnSubmit($event)\" [formGroup]=\"_formGroup\" class=\"aff-form-group\">\n  <ng-container #componentViewContainer></ng-container>\n\n  <ng-container *ngFor=\"let question of _formGroup.questions\">\n    <ng-container *ngIf=\"question.type === 'group'\">\n      <aff-form-group [formGroup]=\"$any(_formGroup.get(question.id))\"></aff-form-group>\n    </ng-container>\n  </ng-container>\n\n  <button style=\"display: none\" type=\"submit\">hidden submit</button>\n</form>\n" }]
        }], ctorParameters: function () {
        return [{ type: ControlFactoryService }, { type: ValidatorFactoryService }, { type: FormRenderService }, { type: i0.Injector }, { type: ControlRegistry }, { type: ActionService, decorators: [{
                        type: Optional
                    }] }, { type: ArrayIndexDirective, decorators: [{
                        type: Optional
                    }] }, { type: i7.HttpClient, decorators: [{
                        type: Optional
                    }] }];
    }, propDecorators: { endpoint: [{
                type: Input
            }], componentViewContainerRef: [{
                type: ViewChild,
                args: ['componentViewContainer', {
                        read: ViewContainerRef,
                        static: true
                    }]
            }], action: [{
                type: Output
            }], submitEvent: [{
                type: Output
            }], formGroup: [{
                type: Input
            }] } });

class FastFormControlComponent {
    constructor(renderService, controlRegistry, injector, actionService, arrayIndex) {
        this.renderService = renderService;
        this.controlRegistry = controlRegistry;
        this.injector = injector;
        this.actionService = actionService;
        this.arrayIndex = arrayIndex;
    }
    ngOnInit() {
        this.createComponent(this.control.question);
    }
    createComponent(question) {
        if (this.controlRegistry.hasItem(question.type)) {
            this.renderService.render(this.componentViewContainerRef, this.control, question, this.controlRegistry.getDefinition(question.type), this.injector, this.actionService);
        }
    }
}
FastFormControlComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: FastFormControlComponent, deps: [{ token: FormRenderService }, { token: ControlRegistry }, { token: i0.Injector }, { token: ActionService, optional: true }, { token: ArrayIndexDirective, optional: true }], target: i0.ɵɵFactoryTarget.Component });
FastFormControlComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "15.2.10", type: FastFormControlComponent, selector: "aff-form-control", inputs: { control: "control" }, viewQueries: [{ propertyName: "componentViewContainerRef", first: true, predicate: ["componentViewContainer"], descendants: true, read: ViewContainerRef, static: true }], ngImport: i0, template: "<ng-container #componentViewContainer></ng-container>\n" });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: FastFormControlComponent, decorators: [{
            type: Component,
            args: [{ selector: 'aff-form-control', template: "<ng-container #componentViewContainer></ng-container>\n" }]
        }], ctorParameters: function () {
        return [{ type: FormRenderService }, { type: ControlRegistry }, { type: i0.Injector }, { type: ActionService, decorators: [{
                        type: Optional
                    }] }, { type: ArrayIndexDirective, decorators: [{
                        type: Optional
                    }] }];
    }, propDecorators: { componentViewContainerRef: [{
                type: ViewChild,
                args: ['componentViewContainer', {
                        read: ViewContainerRef,
                        static: true
                    }]
            }], control: [{
                type: Input
            }] } });

let FastFormRowComponent = class FastFormRowComponent {
    constructor(uiRegistry, renderer, injector, definition, controlRegistry, control, properties, actionService) {
        this.uiRegistry = uiRegistry;
        this.renderer = renderer;
        this.injector = injector;
        this.definition = definition;
        this.controlRegistry = controlRegistry;
        this.control = control;
        this.properties = properties;
        this.actionService = actionService;
        this.codeOnSubmit = new EventEmitter();
    }
    get children() {
        var _a;
        return (_a = this.definition.children) !== null && _a !== void 0 ? _a : [];
    }
    ngOnInit() {
        this.render();
    }
    ngOnChanges() {
        this.render();
    }
    render() {
        this.componentViewContainerRef.clear();
        this.children.filter(question => !question.hidden)
            .forEach(question => {
            this.createComponent(question);
        });
    }
    createComponent(question) {
        var _a;
        if (this.controlRegistry.hasItem(question.type)) {
            const formDefinition = this.controlRegistry.getDefinition(question.type);
            const componentRef = this.uiRegistry.render(this.componentViewContainerRef, this.control, question, formDefinition, this.injector, this.actionService);
            const nativeElement = componentRef.location.nativeElement;
            if (this.properties.size) {
                nativeElement.style.flexBasis = `${(_a = this.properties.size[question.id]) === null || _a === void 0 ? void 0 : _a.percent}%`;
            }
            this.renderer.addClass(componentRef.location.nativeElement, `row_${question.id}`);
        }
    }
};
FastFormRowComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: FastFormRowComponent, deps: [{ token: FormRenderService }, { token: i0.Renderer2 }, { token: i0.Injector }, { token: QuestionDefinition }, { token: ControlRegistry }, { token: FORM_CONTROL }, { token: CONTROL_PROPERTIES }, { token: ActionService, optional: true }], target: i0.ɵɵFactoryTarget.Component });
FastFormRowComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "15.2.10", type: FastFormRowComponent, selector: "aff-form-row", outputs: { codeOnSubmit: "codeOnSubmit" }, viewQueries: [{ propertyName: "componentViewContainerRef", first: true, predicate: ["componentViewContainer"], descendants: true, read: ViewContainerRef, static: true }], usesOnChanges: true, ngImport: i0, template: "<div class=\"aff-form-row\">\n  <ng-container #componentViewContainer></ng-container>\n</div>\n" });
FastFormRowComponent = __decorate([
    Control({
        type: 'row',
        inline: true
    }),
    __metadata("design:paramtypes", [FormRenderService,
        Renderer2,
        Injector,
        QuestionDefinition,
        ControlRegistry,
        AbstractControl, Object, ActionService])
], FastFormRowComponent);
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: FastFormRowComponent, decorators: [{
            type: Component,
            args: [{ selector: 'aff-form-row', template: "<div class=\"aff-form-row\">\n  <ng-container #componentViewContainer></ng-container>\n</div>\n" }]
        }], ctorParameters: function () {
        return [{ type: FormRenderService }, { type: i0.Renderer2 }, { type: i0.Injector }, { type: QuestionDefinition }, { type: ControlRegistry }, { type: i2.AbstractControl, decorators: [{
                        type: Inject,
                        args: [FORM_CONTROL]
                    }] }, { type: undefined, decorators: [{
                        type: Inject,
                        args: [CONTROL_PROPERTIES]
                    }] }, { type: ActionService, decorators: [{
                        type: Optional
                    }] }];
    }, propDecorators: { componentViewContainerRef: [{
                type: ViewChild,
                args: ['componentViewContainer', {
                        read: ViewContainerRef,
                        static: true
                    }]
            }], codeOnSubmit: [{
                type: Output
            }] } });

function ControlFactory() {
    return (target, propertyKey, descriptor) => {
        const controlComponent = target;
        controlComponent[META_COMPONENT_OPTIONS_KEY] = Object.assign(Object.assign({}, controlComponent[META_COMPONENT_OPTIONS_KEY]), { controlFactory: descriptor.value });
    };
}

class FastFormsService {
    constructor(controlFactory, http) {
        this.controlFactory = controlFactory;
        this.http = http;
    }
    createSingleControl(question) {
        return new FastFormControl(question, question.defaultValue);
    }
    createDynamicForm(questions, options) {
        return new FastFormGroup(questions, this.controlFactory, options);
    }
    createHttpForm(endpoint) {
        if (!this.http) {
            throw new Error(`No HttpClient found. Register [HttpClientModule] in your app.`);
        }
        const formGroup = new FastFormGroup([], this.controlFactory);
        this.http.get(endpoint)
            .subscribe(questions => formGroup.setQuestions(questions));
        return formGroup;
    }
}
FastFormsService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: FastFormsService, deps: [{ token: ControlFactoryService }, { token: i7.HttpClient, optional: true }], target: i0.ɵɵFactoryTarget.Injectable });
FastFormsService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: FastFormsService, providedIn: 'any' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: FastFormsService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'any',
                }]
        }], ctorParameters: function () {
        return [{ type: ControlFactoryService }, { type: i7.HttpClient, decorators: [{
                        type: Optional
                    }] }];
    } });

class ControlIdMock {
    constructor(id) {
        this.id = id;
    }
    getId() {
        return this.id;
    }
}

function Validator(options) {
    return (target) => {
        const validatorClass = target;
        validatorClass[META_VALIDATOR_OPTIONS_KEY] = {
            id: options.id,
            type: options.type
        };
    };
}

let FastFormArrayComponent = class FastFormArrayComponent extends BaseFormArrayComponent {
    isControl(data) {
        return data instanceof FormControl;
    }
    isGroup(data) {
        return data instanceof FormGroup;
    }
};
FastFormArrayComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: FastFormArrayComponent, deps: null, target: i0.ɵɵFactoryTarget.Component });
FastFormArrayComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "15.2.10", type: FastFormArrayComponent, selector: "aff-form-array", usesInheritance: true, ngImport: i0, template: "<form class=\"aff-form-array\">\n  <ng-container *ngFor=\"let subForm of formArray.controls; let i = index\">\n    <ng-container *ngIf=\"isControl(subForm)\">\n      <aff-form-control [affArrayIndex]=\"i\" [control]=\"$any(subForm)\"></aff-form-control>\n    </ng-container>\n    <ng-container *ngIf=\"isGroup(subForm)\">\n      <aff-form-group [affArrayIndex]=\"i\" [formGroup]=\"$any(subForm)\"></aff-form-group>\n    </ng-container>\n  </ng-container>\n</form>\n", dependencies: [{ kind: "directive", type: i2$1.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { kind: "directive", type: i2$1.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "directive", type: i2.ɵNgNoValidate, selector: "form:not([ngNoForm]):not([ngNativeValidate])" }, { kind: "directive", type: i2.NgControlStatusGroup, selector: "[formGroupName],[formArrayName],[ngModelGroup],[formGroup],form:not([ngNoForm]),[ngForm]" }, { kind: "directive", type: i2.FormGroupDirective, selector: "[formGroup]", inputs: ["formGroup"], outputs: ["ngSubmit"], exportAs: ["ngForm"] }, { kind: "component", type: FastFormGroupComponent, selector: "aff-form-group", inputs: ["endpoint", "formGroup"], outputs: ["action", "submitEvent"] }, { kind: "component", type: FastFormControlComponent, selector: "aff-form-control", inputs: ["control"] }, { kind: "directive", type: ArrayIndexDirective, selector: "[affArrayIndex]", inputs: ["affArrayIndex"] }] });
FastFormArrayComponent = __decorate([
    Control({
        type: 'array',
        controlType: 'array'
    })
], FastFormArrayComponent);
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: FastFormArrayComponent, decorators: [{
            type: Component,
            args: [{ selector: 'aff-form-array', template: "<form class=\"aff-form-array\">\n  <ng-container *ngFor=\"let subForm of formArray.controls; let i = index\">\n    <ng-container *ngIf=\"isControl(subForm)\">\n      <aff-form-control [affArrayIndex]=\"i\" [control]=\"$any(subForm)\"></aff-form-control>\n    </ng-container>\n    <ng-container *ngIf=\"isGroup(subForm)\">\n      <aff-form-group [affArrayIndex]=\"i\" [formGroup]=\"$any(subForm)\"></aff-form-group>\n    </ng-container>\n  </ng-container>\n</form>\n" }]
        }] });

class FastFormsModule {
    static forRoot(options) {
        var _a, _b, _c;
        return {
            ngModule: FastFormsModule,
            providers: [
                ...((_a = options === null || options === void 0 ? void 0 : options.validators) !== null && _a !== void 0 ? _a : []),
                {
                    provide: VALIDATORS,
                    multi: true,
                    useValue: (_b = options === null || options === void 0 ? void 0 : options.validators) !== null && _b !== void 0 ? _b : [],
                },
                {
                    provide: AFF_CONTROL_COMPONENTS,
                    multi: true,
                    useValue: (_c = options === null || options === void 0 ? void 0 : options.controls) !== null && _c !== void 0 ? _c : [],
                },
            ],
        };
    }
    static forChild(options) {
        var _a, _b, _c;
        return {
            ngModule: FastFormsModule,
            providers: [
                ...((_a = options === null || options === void 0 ? void 0 : options.validators) !== null && _a !== void 0 ? _a : []),
                {
                    provide: VALIDATORS,
                    multi: true,
                    useValue: (_b = options === null || options === void 0 ? void 0 : options.validators) !== null && _b !== void 0 ? _b : [],
                },
                {
                    provide: AFF_CONTROL_COMPONENTS,
                    multi: true,
                    useValue: (_c = options === null || options === void 0 ? void 0 : options.controls) !== null && _c !== void 0 ? _c : [],
                },
            ],
        };
    }
}
FastFormsModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: FastFormsModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
FastFormsModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "15.2.10", ngImport: i0, type: FastFormsModule, declarations: [FastFormGroupComponent,
        FastFormGroupV2Component,
        FastFormArrayComponent,
        FastFormControlComponent,
        FastFormRowComponent,
        ArrayIndexDirective,
        FormRendererDirective,
        FastFormArrayV2Component], imports: [CommonModule,
        ReactiveFormsModule], exports: [FastFormControlComponent,
        FastFormGroupComponent,
        FastFormGroupV2Component,
        FastFormArrayV2Component,
        FastFormRowComponent,
        ArrayIndexDirective,
        FormRendererDirective] });
FastFormsModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: FastFormsModule, providers: [
        {
            provide: AFF_CONTROL_COMPONENTS,
            useValue: [
                FastFormGroupComponent,
                FastFormGroupV2Component,
                FastFormArrayComponent,
                FastFormArrayV2Component,
                FastFormRowComponent
            ],
            multi: true,
        },
    ], imports: [CommonModule,
        ReactiveFormsModule] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: FastFormsModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [
                        CommonModule,
                        ReactiveFormsModule,
                    ],
                    declarations: [
                        FastFormGroupComponent,
                        FastFormGroupV2Component,
                        FastFormArrayComponent,
                        FastFormControlComponent,
                        FastFormRowComponent,
                        ArrayIndexDirective,
                        FormRendererDirective,
                        FastFormArrayV2Component,
                    ],
                    exports: [
                        FastFormControlComponent,
                        FastFormGroupComponent,
                        FastFormGroupV2Component,
                        FastFormArrayV2Component,
                        FastFormRowComponent,
                        ArrayIndexDirective,
                        FormRendererDirective,
                    ],
                    providers: [
                        {
                            provide: AFF_CONTROL_COMPONENTS,
                            useValue: [
                                FastFormGroupComponent,
                                FastFormGroupV2Component,
                                FastFormArrayComponent,
                                FastFormArrayV2Component,
                                FastFormRowComponent
                            ],
                            multi: true,
                        },
                    ],
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { AFF_CONTROL_COMPONENTS, ActionControl, ActionControlFactory, ActionGroupFactory, ActionService, ArrayIndexDirective, BaseFormArrayComponent, BaseFormControlComponent, BaseFormGroupComponent, CONTROL_ID, CONTROL_PROPERTIES, Control, ControlFactory, ControlFactoryService, ControlIdMock, ControlWrapperKey, FORM_CONTROL, FastFormArray, FastFormArrayV2Component, FastFormBuilder, FastFormControl, FastFormControlComponent, FastFormGroup, FastFormGroupComponent, FastFormGroupV2Component, FastFormRowComponent, FastFormsModule, FastFormsService, FormRendererDirective, QuestionDefinition, Validator, hasControlWrapper };
//# sourceMappingURL=ngx-fast-forms-core.mjs.map
