import { AbstractControl, AbstractControlOptions, AsyncValidatorFn, FormControlOptions, ValidatorFn } from '@angular/forms';
import { ValidationOptions } from './model';
import { FastFormBuilder } from './service/fast-form-builder';
export type TypedQuestion<PropType = unknown> = Omit<GeneralQuestion<PropType>, 'type'> & {
    type: string;
};
export interface GeneralQuestion<PropType = unknown> {
    type?: string;
    id?: string;
    label?: string;
    hidden?: boolean;
    disabled?: boolean;
    validation?: ValidationOptions;
    properties?: PropType;
}
export interface InitialValue<T = unknown> {
    defaultValue?: T;
}
export type ControlBuilderDefinition<T = unknown> = TypedQuestion<T> & FormControlOptions;
export type GroupBuilderDefinition<T = unknown> = GeneralQuestion<T> & AbstractControlOptions;
export type ArrayBuilderDefinition<T = unknown> = GeneralQuestion<T> & AbstractControlOptions;
export interface ControlFactoryOptions {
    fb: FastFormBuilder;
    validators: ValidatorFn[] | undefined | null;
    asyncValidators: AsyncValidatorFn[] | undefined | null;
}
export type FactoryQuestionDefinition<PropType = unknown, DefaultType = unknown> = TypedQuestion<PropType> & FormControlOptions & InitialValue<DefaultType>;
export type ControlFactoryMethod = (question: FactoryQuestionDefinition, opts: ControlFactoryOptions) => AbstractControl;
